package WebService::dSS;

# ABSTRACT: Module to interface with the digitalSTROM Server JSON API

use Moose;
use MooseX::FollowPBP;
use Moose::Util::TypeConstraints;
use Carp;
use URI;
use URI::QueryParam;
use JSON::XS;
use LWP::UserAgent;
use WebService::dSS::Apartment;
use MooseX::Types::URI qw(Uri);
use namespace::autoclean;

=head1 SYNOPSIS

	use WebService::dSS;

	my $api = WebService::dSS->new({
		url => 'https://localhost:8080/',
	});

	my $result = $api->call({
		method => 'apartment.getStructure',
	});

	$result = $api->call({
		method => 'zone.turnOff',
		param  => {
			id      => 716,
			groupID => 1
		}
	});

	my $apartment = $api->get_apartment();

=head1 DESCRIPTION

WebService::dSS does provide a Perl module interface to the digitalSTROM Server
JSON API.

=head1 METHODS

=head2 new

Constructor, creates new instance.

=head3 Parameters

This method expects its parameters as a hash reference.

=over

=item url

Base URL of the dSS.

=item username

Username for login. Optional, but some operations may fail.

=item password

Password for the login. Optional, but some operations may fail.

=item ua

The L<LWP::UserAgent|LWP::UserAgent> instance to use.

=back

=cut


=head2 get_url

Getter for the C<url> attribute with the dSS base URL.

=head3 Result

The C<url> attribute's value.

=cut

has 'url' => (
	is      => 'ro',
	isa     => Uri,
	coerce  => 1,
	default => sub { URI->new( 'https://localhost:8080/' ) },
);


=head2 get_ua

Getter for the C<ua> attribute.

=head3 Result

The C<ua> attribute's value.

=cut

has 'ua' => (
	is       => 'ro',
	isa      => 'LWP::UserAgent',
	lazy     => 1,
	default  => sub { LWP::UserAgent->new( cookie_jar => {} ) },
);


=head2 get_username

Getter for the C<username> attribute.

=head3 Result

The C<username> attribute's value.

=cut

has 'username' => (
	is       => 'ro',
	isa      => 'Str',
	required => 0,
);


=head2 get_password

Getter for the C<password> attribute.

=head3 Result

The C<password> attribute's value.

=cut

has 'password' => (
	is       => 'ro',
	isa      => 'Str',
	required => 0,
);


has '_timeout' => (
	is       => 'rw',
	isa      => 'Int',
	init_arg => undef,
);


has '_last_login' => (
	is       => 'rw',
	isa      => 'Int',
	init_arg => undef,
);


=head2 call

Call a method on the dSS.

=head3 Parameters

This method expects its parameters as a hash reference.

=over

=item method

Name of the method which should be called.

=item param

Parameters that should be passed in the call.

=item no_login

Flag to indicate that no login should be attempted.

=back

=head3 Result

The result object (usually a hash reference or C<undef>) on success, an
exception otherwise.

=cut

sub call {
	my ($self, $arg_ref) = @_;

	unless (defined $arg_ref && ref $arg_ref eq 'HASH') {
		croak("No parameters passed or not a hash reference");
	}

	my $method = $arg_ref->{method};
	unless (defined $method && $method ne '') {
		croak("Required 'method' parameter not passed");
	}
	my @method = split(/\./, $method);

	my $param = $arg_ref->{param};
	if (defined $param && ref $param ne 'HASH') {
		croak("Optional 'param' parameter passed, but not a hash reference");
	}

	# Login if session likely timed out
	unless ($arg_ref->{no_login}) {
		if (! defined $self->_get_timeout() || ! defined $self->_get_last_login()
				|| (time() >= $self->_get_timeout() + $self->_get_last_login())) {
			$self->login();
		}
	}

	# Construct URL
	my $url = $self->get_url()->clone();
	$url->path(join('/', ($url->path_segments, 'json', @method)));
	if (defined $param) {
		$url->query_form_hash($param)
	}

	# Call method
	my $result = $self->get_ua()->get($url);

	if ($result->is_success()) {
		my $result_object = JSON::XS->new->utf8->decode($result->content());
		if ($result_object->{ok}) {
			return $result_object->{result};
		}
		else {
			confess("Call failed: ".$result_object->{message});
		}
	}
	else {
		confess('Failed to execute call: '.$result->status_line());
	}
}


=head2 login

Login to the dSS if username/password were given.

=head3 Result

Nothing on success, an exception otherwise.

=cut

sub login {
	my ($self) = @_;

	my $username = $self->get_username();
	my $password = $self->get_password();

	if (defined $username && $username ne '' && defined $password && $password ne '') {
		$self->call({
			method   => 'system.login',
			param    => {
				user     => $username,
				password => $password,
			},
			no_login => 1
		});
		$self->_set_last_login(time());

		# Get session timeout
		if (! defined $self->_get_timeout()) {
			my $result = $self->call({
				method   => 'property.getInteger',
				param    => {
					path => '/config/subsystems/WebServer/sessionTimeoutMinutes',
				},
				no_login => 1
			});
			$self->_set_timeout(60 * $result->{value} * 0.75); # * 0.75 to be on the safer side
		}
	}
	return;
}


=head2 get_apartment

Getter for the apartment which is controlled by the dSS.

=head3 Result

A L<WebService::dSS::Apartment|WebService::dSS::Apartment> instance which
represents the apartment.

=cut

sub get_apartment {
	my ($self) = @_;

	return WebService::dSS::Apartment->new({
		api => $self,
	});
}


__PACKAGE__->meta()->make_immutable();


=head1 SEE ALSO

=over 4

=item L<http://www.digitalstrom.org/>

The official home page of the digitalSTROM project.

=item L<http://developer.digitalstrom.org/redmine/wiki/dss>

digitalSTROM Server wiki.

=back

=cut
