package WebService::dSS::Apartment;

# ABSTRACT: Represent an apartment

use Moose;
use MooseX::FollowPBP;

use Moose::Util::TypeConstraints;
use Carp;
use WebService::dSS::Apartment::Zone;

use namespace::autoclean;


=head1 SYNOPSIS

	use WebService::dSS;
	use WebService::dSS::Apartment;

	my $apartment = WebService::dSS::Apartment->new({
		api => WebService::dSS->new({
			url => 'https://localhost:8080/',
		}),
	});

	$apartment->turn_on();
	$apartment->turn_off();

	my @zones = $apartment->get_zones();

=head1 DESCRIPTION

WebService::dSS::Apartment does represent an apartment in a digitalSTROM
installation.

=head1 METHODS

=head2 new

Constructor, creates new instance.

=head3 Parameters

This method expects its parameters as a hash reference.

=over

=item api

Reference to a L<WebService::dSS|WebService::dSS> instance.

=back

=cut

has 'api' => (
	is       => 'ro',
	isa      => 'WebService::dSS',
	required => 1,
);


=head2 get_name_base_params

Getter for the parameters used when reading/writing the name. This method is
required by the L<WebService::dSS::Role::Nameable|WebService::dSS::Role::Nameable>
role.

=head3 Result

A hash reference with the base parameters.

=cut

has 'name_base_params' => (
	is       => 'ro',
	isa      => 'HashRef',
	init_arg => undef,
	lazy     => 1,
	default  => sub {
		return {
			read => {
				param  => {},
				method => 'apartment.getName'
			},
			write => {
				param  => {},
				method => 'apartment.setName'
			}
		}
	}
);


=head2 get_switch_base_params

Getter for the parameters used when switching the apartment on or off. This method is
required by the L<WebService::dSS::Role::Switchable|WebService::dSS::Role::Switchable>
role.

=head3 Result

A hash reference with the base parameters.

=cut


has 'switch_base_params' => (
	is       => 'ro',
	isa      => 'HashRef',
	required => 0,
	default  => sub {
		return {
			on => {
				param  => {},
				method => 'apartment.turnOn'
			},
			off => {
				param  => {},
				method => 'apartment.turnOff'
			}
		};
	}
);


=head2 get_zones

Get the zones of the apartment.

=head3 Parameters

This method expects its parameters as a hash reference.

=over

=item include_all

Include zone which contains all devices (i.e. zone 0).

=back

=head3 Result

A list with the zones, as
L<WebService::dSS::Apartment::Zone|WebService::dSS::Apartment::Zone> instances.

=cut

sub get_zones {
	my ($self, $arg_ref) = @_;

	unless (defined $arg_ref && ref $arg_ref eq 'HASH') {
		$arg_ref = {}
	}

	my $zones = $self->get_api()->call({
		method => 'apartment.getStructure',
	})->{apartment}->{zones};

	my @zones = @{$zones};

	# Filter out zone with all devices
	unless ($arg_ref->{include_all}) {
		@zones = grep {
			$_->{id} != 0
		} @zones;
	}

	return map {
		WebService::dSS::Apartment::Zone->new({
			apartment => $self,
			id        => $_->{id},
		});
	} @zones;
}


=head1 ROLES

WebService::dSS::Apartment::Zone does the following roles:

=over

=item *

L<WebService::dSS::Role::Nameable|WebService::dSS::Role::Nameable>

=item *

L<WebService::dSS::Role::Switchable|WebService::dSS::Role::Switchable>

=back

=cut

with 'WebService::dSS::Role::Nameable';
with 'WebService::dSS::Role::Switchable';


__PACKAGE__->meta()->make_immutable();
