package WebService::dSS::Apartment::Zone;

# ABSTRACT: Represent a zone in an apartment


use Moose;
use MooseX::FollowPBP;
use Moose::Util::TypeConstraints;
use List::Util qw(first);
use namespace::autoclean;
use WebService::dSS::Apartment::Zone::Group;

use Carp;


=head1 SYNOPSIS

	use WebService::dSS;
	use WebService::dSS::Apartment::Zone;

	my $apartment = WebService::dSS->new({
		url => 'https://localhost:8080/',
	})->get_apartment();

	my $zone = WebService::dSS::Apartment::Zone->new({
		apartment => $apartment,
		id        => 1,
	});

	$zone->turn_on();
	$zone->turn_off();

	my @groups = $zone->get_groups();

=head1 DESCRIPTION

WebService::dSS::Apartment::Zone does represent a zone of an apartment in a
digitalSTROM installation.

=head1 METHODS

=head2 new

Constructor, creates new instance.

=head3 Parameters

This method expects its parameters as a hash reference.

=over

=item apartment

The apartment this zone is a part of.

=item id

Id of the zone.

=back

=cut

has 'apartment' => (
	is       => 'ro',
	isa      => 'WebService::dSS::Apartment',
	required => 1,
	handles  => ['get_api'],
);

=head2 get_id

Getter for the C<id> attribute.

=head3 Result

The C<id> attribute's value.

=cut

has 'id' => (
	is       => 'ro',
	isa      => 'Int',
	required => 1,
);


=head2 get_name_base_params

Getter for the parameters used when reading/writing the name. This method is
required by the L<WebService::dSS::Role::Nameable|WebService::dSS::Role::Nameable>
role.

=head3 Result

A hash reference with the base parameters.

=cut

has 'name_base_params' => (
	is       => 'ro',
	isa      => 'HashRef',
	init_arg => undef,
	lazy     => 1,
	default  => sub {
		my ($self) = @_;
		return {
			read => {
				param  => {
					id => $self->get_id(),
				},
				method => 'zone.getName'
			},
			write => {
				param  => {
					id => $self->get_id(),
				},
				method => 'zone.setName'
			}
		}
	}
);


=head2 get_switch_base_params

Getter for the parameters used when switching the zone on or off. This method is
required by the L<WebService::dSS::Role::Switchable|WebService::dSS::Role::Switchable>
role.

=head3 Result

A hash reference with the base parameters.

=cut


has 'switch_base_params' => (
	is       => 'ro',
	isa      => 'HashRef',
	init_arg => undef,
	default  => sub {
		my ($self) = @_;
		return {
			on => {
				param  => {
					id => $self->get_id(),
				},
				method => 'zone.turnOn'
			},
			off => {
				param  => {
					id => $self->get_id(),
				},
				method => 'zone.turnOff'
			}
		};
	}
);


=head2 get_groups

Get the groups of this zone.

=head3 Parameters

This method expects its parameters as a hash reference.

=over

=item include_empty

Include groups which have no devices. Defaults to false.

=item include_broadcast

Include broadcast group. Defaults to false.

=back

=head3 Result

A list with the groups as
L<WebService::dSS::Apartment::Zone::Group|WebService::dSS::Apartment::Zone::Group>
instances.

=cut

sub get_groups {
	my ($self, $arg_ref) = @_;

	unless (defined $arg_ref && ref $arg_ref eq 'HASH') {
		$arg_ref = {}
	}

	my $result = $self->get_api()->call({
		method => 'apartment.getStructure',
		param  => {},
	});
	my $zone = first { $_->{id} eq $self->get_id() } @{$result->{apartment}->{zones}};
	my @groups = @{$zone->{groups}};

	# Filter out empty groups
	unless ($arg_ref->{include_empty}) {
		@groups = grep {
			scalar @{$_->{devices}} > 0
		} @groups;
	}

	# Filter out broadcast group
	unless ($arg_ref->{include_broadcast}) {
		@groups = grep {
			$_->{id} != 0
		} @groups;
	}

	# Create group objects
	return map {
		WebService::dSS::Apartment::Zone::Group->new({
			id   => $_->{id},
			name => $_->{name},
			zone => $self,
		})
	} @groups;
}


=head1 ROLES

WebService::dSS::Apartment::Zone does the following roles:

=over

=item *

L<WebService::dSS::Role::Nameable|WebService::dSS::Role::Nameable>

=item *

L<WebService::dSS::Role::Switchable|WebService::dSS::Role::Switchable>

=back

=cut

with 'WebService::dSS::Role::Nameable';
with 'WebService::dSS::Role::Switchable';

__PACKAGE__->meta()->make_immutable();
