package WebService::dSS::Apartment::Zone::Group;

# ABSTRACT: Represent a group in a zone of an apartment

use Moose;
use MooseX::FollowPBP;
use Moose::Util::TypeConstraints;
use namespace::autoclean;
use Carp;


=head1 SYNOPSIS

	use WebService::dSS::Apartment::Zone::Group;

	my $apartment = WebService::dSS->new({
		url => 'https://localhost:8080/',
	})->get_apartment();

	my $zone = WebService::dSS::Apartment::Zone->new({
		apartment => $apartment,
		id        => 1,
	});

	my $group = WebService::dSS::Apartment::Zone::Group->new({
		zone => $zone,
		id   => 1,
		name => 'yellow',
	});

	$group->turn_on();
	$group->turn_off();

=head1 DESCRIPTION

WebService::dSS::Apartment::Zone::Group does represent a group in a zone of an
apartment.

=head1 METHODS

=head2 new

Constructor, creates new instance.

=head3 Parameters

This method expects its parameters as a hash reference.

=over

=item zone

The zone this group belongs to.

=item id

Id of this group within the zone.

=item name

Optional name of the group. Will be retrieved from the dSS if not passed.

=back

=head2 get_zone

Getter for the C<zone> attribute.

=head3 Result

The C<zone> attribute's value.

=cut

has 'zone' => (
	is       => 'ro',
	isa      => 'WebService::dSS::Apartment::Zone',
	required => 1,
	handles  => ['get_api'],
);


=head2 get_id

Getter for the C<id> attribute.

=head3 Result

The C<id> attribute's value.

=cut

has 'id' => (
	is       => 'ro',
	isa      => 'Int',
	required => 1,
);


=head2 get_switch_base_params

Getter for the parameters used when switching the group on or off. This method is
required by the L<WebService::dSS::Role::Switchable|WebService::dSS::Role::Switchable>
role.

=head3 Result

A hash reference with the base parameters.

=cut


has 'switch_base_params' => (
	is       => 'ro',
	isa      => 'HashRef',
	init_arg => undef,
	default  => sub {
		my ($self) = @_;
		return {
			on => {
				param  => {
					id      => $self->get_zone()->get_id(),
					groupID => $self->get_id(),
				},
				method => 'zone.turnOn'
			},
			off => {
				param  => {
					id      => $self->get_zone()->get_id(),
					groupID => $self->get_id(),
				},
				method => 'zone.turnOff'
			}
		};
	}
);


=head2 get_name_base_params

Getter for the parameters used when reading the name. This method is
required by the L<WebService::dSS::Role::Named|WebService::dSS::Role::Named>
role.

=head3 Result

A hash reference with the base parameters.

=cut

has 'name_base_params' => (
	is       => 'ro',
	isa      => 'HashRef',
	init_arg => undef,
	lazy     => 1,
	default  => sub {
		my ($self) = @_;
		return {
			read => {
				param    => {
					path  => '/',
					query => '/apartment/zones/zone'.$self->get_zone()->get_id()
								.'/groups/group'.$self->get_id().'(group,name)',
				},
				method   => 'property.query',
				callback => sub {
					my ($result) = @_;
					return $result->{'group'.$self->get_id()}->[0]->{name};
				}
			},
		}
	}
);


=head1 ROLES

WebService::dSS::Apartment::Zone::Group does the following roles:

=over

=item *

L<WebService::dSS::Role::Switchable|WebService::dSS::Role::Switchable>

=item *

L<WebService::dSS::Role::Named|WebService::dSS::Role::Named>

=back

=cut

with 'WebService::dSS::Role::Switchable';
with 'WebService::dSS::Role::Named';


__PACKAGE__->meta()->make_immutable();
