package WebService::dSSTest;
use base qw(WebService::dSS::TestBase);

use strict;
use warnings;

use Carp;
use Test::More;
use Test::Exception;
use Test::Deep;
use URI;
use JSON::XS;
use WebService::dSS;

sub new_test : Test(8) {
	my ($self) = @_;

	# Default URI
	my $api = WebService::dSS->new();
	isa_ok($api, 'WebService::dSS', 'instance created');
	is($api->get_url(), 'https://localhost:8080/', 'default URL in use');

	# Pass URI
	$api = WebService::dSS->new({
		url => $self->get_base_url('0'),
	});
	isa_ok($api, 'WebService::dSS', 'instance created');
	is($api->get_url(), $self->get_base_url('0'), 'URL set');

	# Pass URI as string
	$api = WebService::dSS->new({
		url => 'https://www.example.com:8080/'
	});
	isa_ok($api, 'WebService::dSS', 'instance created');
	is($api->get_url(), 'https://www.example.com:8080/', 'URL set use');

	# Pass username/password
	$api = WebService::dSS->new({
		url      => $self->get_base_url('0'),
		username => 'foo',
		password => 'bar',
	});
	is($api->get_username(), 'foo');
	is($api->get_password(), 'bar');
}


sub call_test : Test(4) {
	my ($self) = @_;

	my $api = WebService::dSS->new({
		url => $self->get_base_url('0'),
	});

	# Complex result
	my $result = $api->call({
		method => 'apartment.getStructure',
		param  => {},
	});
	sub check_devices {
		sub { ref $_[0] eq 'ARRAY' ? 1 : 0 }
	}
	sub check_groups {
		sub { ref $_[0] eq 'ARRAY' ? 1 : 0 }
	}
	cmp_deeply($result, {
		apartment => {
			zones => [
				{
					id        => 0,
					name      => '',
					isPresent => JSON::XS::true,
					devices   => code(\&check_devices),
					groups    => code(\&check_groups),
				},
				{
					id                 => 1,
					name               => '',
					isPresent          => JSON::XS::true,
					firstZoneOnDSMeter => '3504175fe0000000ffc00010',
					devices   => code(\&check_devices),
					groups    => code(\&check_groups),
				}
			]
		}
	}, 'result looks fine');

	# Parameters
	$result = $api->call({
		method => 'apartment.foo',
		param  => {
			asdf => 1,
		},
	});
	is_deeply($result, {a => 1}, 'param passed');

	# Unknown method
	throws_ok(sub {
		$api->call({
			method => 'apartment.foo',
		});
	}, qr{Failed to execute call:}, 'method not known');

	# Call fails
	throws_ok(sub {
		$api->call({
			method => 'apartment.failure',
		});
	}, qr{Call failed for some reason}, 'call failed');
}


sub login_for_call_test : Test(6) {
	my ($self) = @_;

	# Do login if username/password passed
	my $api = WebService::dSS->new({
		url      => $self->get_base_url('0'),
		username => 'dssadmin',
		password => 'dssadmin',
		ua       => $self->get_fake_ua(),
	});
	is($api->_get_timeout(), undef, 'no timeout known');
	is($api->_get_last_login(), undef, 'no last login time');
	$api->call({
		method => 'apartment.getStructure',
		param  => {},
	});
	is($api->_get_timeout(), 15 * 60 * 0.75, 'no timeout known');
	my $time       = time();
	my $last_login = $api->_get_last_login();
	ok(
		$last_login > $time - 10 && $last_login < $time + 10,
		'last login time set'
	);

	# No other login if session has not expired
	$api->_set_last_login($last_login - 60);
	$api->call({
		method => 'apartment.getStructure',
		param  => {},
	});
	is($api->_get_last_login(), $last_login - 60, 'no other login');

	# Login again after timeout
	$api->_set_last_login($time - 3600);
	$api->call({
		method => 'apartment.getStructure',
		param  => {},
	});
	ok($api->_get_last_login() >= $time, 'logged in again');
}


sub get_apartment_test : Test(1) {
	my ($self) = @_;

	my $api = WebService::dSS->new({
		url => $self->get_base_url('0'),
	});

	my $apartment = $api->get_apartment();
	isa_ok($apartment, 'WebService::dSS::Apartment', 'apartment retrieved');
}


sub login_test : Test(3) {
	my ($self) = @_;

	my $api = WebService::dSS->new({
		url      => $self->get_base_url('0'),
		username => 'dssadmin',
		password => 'dssadmin',
		ua       => $self->get_fake_ua(),
	});

	lives_ok(sub { $api->login() }, 'login worked');
	is($api->_get_timeout, 15 * 60 * 0.75, 'timeout set');
	my $time = time();
	ok(
		$api->_get_last_login() > $time - 10 && $api->_get_last_login() < $time + 10,
		'last login time set'
	);
}


1;
